@@ -23,6 +23,13 @@ Developed by Stackmate India in 2021.
 //!
 //!
 //! ### Tor controls are in BETA. Use with caution.
+//! 
+
+use std::alloc::System;
+
+#[global_allocator]
+static A: System = System;
+
 use std::ffi::{CStr, CString};
 use std::os::raw::c_char;
 use std::str;
@@ -48,8 +55,7 @@ use crate::wallet::psbt;
 pub mod network;
 use crate::network::fees;
 use crate::network::height;
-
-// use crate::network::tor;
+use crate::network::tor;
 
 /// Generates a mnemonic phrase of a given length. Defaults to 24 words.
 /// A master xprv is created from the mnemonic and passphrase.
@@ -895,54 +901,54 @@ pub unsafe extern "C" fn days_to_blocks(days: *const c_char) -> *mut c_char {
 /// # Safety
 /// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
-// #[no_mangle]
-// pub unsafe extern "C" fn tor_start(tmp_path: *mut c_char) -> *mut c_char {
-//   let tmp_path_cstr = CStr::from_ptr(tmp_path);
-//   let tmp_path: &str = match tmp_path_cstr.to_str() {
-//     Ok(string) => string,
-//     Err(_) => "/tmp",
-//   };
+#[no_mangle]
+pub unsafe extern "C" fn tor_start(tmp_path: *mut c_char) -> *mut c_char {
+  let tmp_path_cstr = CStr::from_ptr(tmp_path);
+  let tmp_path: &str = match tmp_path_cstr.to_str() {
+    Ok(string) => string,
+    Err(_) => "/tmp",
+  };
 
-//   let control_key = tor::start(tmp_path);
-//   CString::new(control_key).unwrap().into_raw()
-// }
+  let control_key = tor::start(tmp_path);
+  CString::new(control_key).unwrap().into_raw()
+}
 
 /// Get bootstrap progress from tor daemon. Wait ~1s after calling tor_start() before calling this.
 /// BETA: Careful with this.
 /// # Safety
 /// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
-// #[no_mangle]
-// pub unsafe extern "C" fn tor_progress(control_key: *mut c_char) -> *mut c_char {
-//   let control_key_cstr = CStr::from_ptr(control_key);
-//   let control_key: &str = match control_key_cstr.to_str() {
-//     Ok(string) => string,
-//     Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
-//   };
-//   match tor::bootstrap_progress(control_key){
-//     Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
-//     Err(e) => e.c_stringify()
-//   }
-// }
+#[no_mangle]
+pub unsafe extern "C" fn tor_progress(control_key: *mut c_char) -> *mut c_char {
+  let control_key_cstr = CStr::from_ptr(control_key);
+  let control_key: &str = match control_key_cstr.to_str() {
+    Ok(string) => string,
+    Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
+  };
+  match tor::bootstrap_progress(control_key){
+    Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
+    Err(e) => e.c_stringify()
+  }
+}
 
 /// Shutdown tor daemon.
 /// BETA: Careful with this.
 /// # Safety
 /// - This function is unsafe because it dereferences and a returns raw pointer.
 /// - ENSURE that result is passed into cstring_free(ptr: *mut c_char) after use.
-// #[no_mangle]
-// pub unsafe extern "C" fn tor_stop(control_key: *mut c_char) -> *mut c_char {
-//   let control_key_cstr = CStr::from_ptr(control_key);
-//   let control_key: &str = match control_key_cstr.to_str() {
-//     Ok(string) => string,
-//     Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
-//   };
+#[no_mangle]
+pub unsafe extern "C" fn tor_stop(control_key: *mut c_char) -> *mut c_char {
+  let control_key_cstr = CStr::from_ptr(control_key);
+  let control_key: &str = match control_key_cstr.to_str() {
+    Ok(string) => string,
+    Err(_) => return S5Error::new(ErrorKind::Input, "Control-Key").c_stringify(),
+  };
 
-//   match tor::shutdown(control_key){
-//     Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
-//     Err(e) => e.c_stringify()
-//   }
-// }
+  match tor::shutdown(control_key){
+    Ok(result) => CString::new(result.to_string()).unwrap().into_raw(),
+    Err(e) => e.c_stringify()
+  }
+}
 
 /// After using any other function, pass the output pointer into cstring_free(ptr: *mut c_char) to clear memory.
 /// ALWAYS use this in combination with any other function.
